@@ -1,6 +1,6 @@
 package Dancer2::Core::Cookie;
 # ABSTRACT: A cookie representing class
-$Dancer2::Core::Cookie::VERSION = '0.161000';
+$Dancer2::Core::Cookie::VERSION = '0.161000_01';
 use Moo;
 use URI::Escape;
 use Dancer2::Core::Types;
@@ -12,7 +12,7 @@ sub to_header {
     my $self   = shift;
     my $header = '';
 
-    my $value = join( '&', map { uri_escape($_) } $self->value );
+    my $value = join( '&', map uri_escape($_), $self->value );
     my $no_httponly = defined( $self->http_only ) && $self->http_only == 0;
 
     my @headers = $self->name . '=' . $value;
@@ -106,7 +106,7 @@ Dancer2::Core::Cookie - A cookie representing class
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 SYNOPSIS
 
