@@ -1,7 +1,7 @@
 # ABSTRACT: Dancer2's Domain Specific Language (DSL)
 
 package Dancer2::Core::DSL;
-$Dancer2::Core::DSL::VERSION = '0.161000';
+$Dancer2::Core::DSL::VERSION = '0.161000_01';
 use Moo;
 use Carp;
 use Class::Load 'load_class';
@@ -193,7 +193,7 @@ sub prefix {
       : $app->lexical_prefix(@_);
 }
 
-sub halt { shift->app->halt }
+sub halt { shift->app->halt(@_) }
 
 sub del     { shift->_normalize_route( [qw/delete  /], @_ ) }
 sub get     { shift->_normalize_route( [qw/get head/], @_ ) }
@@ -447,7 +447,7 @@ Dancer2::Core::DSL - Dancer2's Domain Specific Language (DSL)
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 FUNCTIONS
 
