@@ -1,12 +1,14 @@
 package Dancer2::Core::Error;
 # ABSTRACT: Class representing fatal errors
-$Dancer2::Core::Error::VERSION = '0.161000';
+$Dancer2::Core::Error::VERSION = '0.161000_01';
 use Moo;
 use Carp;
 use Dancer2::Core::Types;
 use Dancer2::Core::HTTP;
 use Data::Dumper;
 use Dancer2::FileUtils qw/path open_file/;
+use Sub::Quote;
+use Safe::Isa;
 
 has app => (
     is        => 'ro',
@@ -68,7 +70,7 @@ sub _build_error_template {
     # look for a template named after the status number.
     # E.g.: views/404.tt  for a TT template
     return $self->status
-      if -f $self->app->engine('template')
+      if -f $self->app->template_engine
           ->view_pathname( $self->status );
 
     return;
@@ -101,29 +103,25 @@ sub default_error_page {
 
     require Template::Tiny;
 
-    my $uri_base = $self->has_app ?
+    my $uri_base = $self->has_app && $self->app->has_request ?
         $self->app->request->uri_base : '';
 
-    my $message = $self->message;
-    if ( $self->show_errors && $self->exception) {
-        $message .= "\n" . $self->exception;
-    }
-
     my $opts = {
         title    => $self->title,
         charset  => $self->charset,
-        content  => $message,
+        content  => $self->show_errors ? $self->full_message : $self->message || 'Wooops, something went wrong',
         version  => Dancer2->VERSION,
         uri_base => $uri_base,
     };
 
     Template::Tiny->new->process( \<<"END_TEMPLATE", $opts, \my $output );
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
-<html>
+<!DOCTYPE html>
+<html lang="en">
 <head>
-<title>[% title %]</title>
-<link rel="stylesheet" href="[% uri_base %]/css/error.css" />
-<meta http-equiv="Content-type" content="text/html; charset='[% charset %]'" />
+  <meta charset="[% charset %]">
+  <meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=yes">
+  <title>[% title %]</title>
+  <link rel="stylesheet" href="[% uri_base %]/css/error.css">
 </head>
 <body>
 <h1>[% title %]</h1>
@@ -163,24 +161,23 @@ sub full_message {
 
 has serializer => (
     is        => 'ro',
-    isa       => Maybe[ConsumerOf ['Dancer2::Core::Role::Serializer']],
+    isa       => Sub::Quote::quote_sub(q{
+        $_[0]
+            ? $_[0]->$_DOES('Dancer2::Core::Role::Serializer')
+            : 1;
+    }),
     builder   => '_build_serializer',
 );
 
 sub _build_serializer {
     my ($self) = @_;
 
-    $self->has_app
-        and return $self->app->engine('serializer');
+    $self->has_app && $self->app->has_serializer_engine
+        and return $self->app->serializer_engine;
 
     return;
 }
 
-has session => (
-    is  => 'ro',
-    isa => ConsumerOf ['Dancer2::Core::Role::Session'],
-);
-
 sub BUILD {
     my ($self) = @_;
 
@@ -264,6 +261,7 @@ sub _build_content {
                 }
             );
         };
+        $@ && $self->app->engine('logger')->log( warning => $@ );
 
         # return rendered content unless there was an error.
         return $content if defined $content;
@@ -285,6 +283,7 @@ sub _build_content {
                 }
             );
         };
+        $@ && $self->app->engine('logger')->log( warning => $@ );
 
         # return rendered content unless there was an error.
         return $content if defined $content;
@@ -308,26 +307,25 @@ sub throw {
     $self->response->status( $self->status );
     $self->response->content_type( $self->content_type );
     $self->response->content($message);
-    $self->response->encode_content;
 
     $self->has_app &&
         $self->app->execute_hook('core.error.after', $self->response);
 
-    $self->response->halt(1);
+    $self->response->is_halted(1);
     return $self->response;
 }
 
 sub backtrace {
     my ($self) = @_;
 
-    my $message = $self->exception ? $self->exception : $self->message;
-    $message =
-      qq|<pre class="error">| . _html_encode( $message ) . "</pre>";
-
-    if ( $self->exception && !ref($self->exception) ) {
-        $message .= qq|<pre class="error">|
-                 . _html_encode($self->exception) . "</pre>";
+    my $message = $self->message;
+    if ($self->exception) {
+        $message .= "\n" if $message;
+        $message .= $self->exception;
     }
+    $message ||= 'Wooops, something went wrong';
+
+    $message = '<pre class="error">' . _html_encode($message) . '</pre>';
 
     # the default perl warning/error pattern
     my ( $file, $line ) = ( $message =~ /at (\S+) line (\d+)/ );
@@ -340,7 +338,7 @@ sub backtrace {
     return $message unless ( $file and $line );
 
     # file and line are located, let's read the source Luke!
-    my $fh = open_file( '<', $file ) or return $message;
+    my $fh = eval { open_file( '<', $file ) } or return $message;
     my @lines = <$fh>;
     close $fh;
 
@@ -396,9 +394,14 @@ sub dumper {
 
     #use Data::Dumper;
     my $dd = Data::Dumper->new( [ \%data ] );
-    $dd->Terse(1)->Quotekeys(0)->Indent(1);
-    my $content = $dd->Dump();
-    $content =~ s{(\s*)(\S+)(\s*)=>}{$1<span class="key">$2</span>$3 =&gt;}g;
+    my $hash_separator = '  @@!%,+$$#._(--  '; # Very unlikely string to exist already
+    my $prefix_padding = '  #+#+@%.,$_-!((  '; # Very unlikely string to exist already
+    $dd->Terse(1)->Quotekeys(0)->Indent(1)->Sortkeys(1)->Pair($hash_separator)->Pad($prefix_padding);
+    my $content = _html_encode( $dd->Dump );
+    $content =~ s/^.+//;   # Remove the first line
+    $content =~ s/\n.+$//; # Remove the last line
+    $content =~ s/^\Q$prefix_padding\E  //gm; # Remove the padding
+    $content =~ s{^(\s*)(.+)\Q$hash_separator}{$1<span class="key">$2</span> =&gt; }gm;
     if ($censored) {
         $content
           .= "\n\nNote: Values of $censored sensitive-looking keys hidden\n";
@@ -409,31 +412,20 @@ sub dumper {
 sub environment {
     my ($self) = @_;
 
-    my $request = $self->has_app ? $self->app->request : 'TODO';
-    my $r_env = {};
-    $r_env = $request->env if defined $request;
+    my $stack = $self->get_caller;
+    my $settings = $self->has_app && $self->app->settings;
+    my $session = $self->has_app && $self->app->_has_session && $self->app->session->data;
+    my $env = $self->has_app && $self->app->has_request && $self->app->request->env;
 
-    my $env =
-        qq|<div class="title">Environment</div><pre class="content">|
-      . dumper($r_env)
-      . "</pre>";
-    my $settings =
-        qq|<div class="title">Settings</div><pre class="content">|
-      . dumper( $self->app->settings )
-      . "</pre>";
-    my $source =
-        qq|<div class="title">Stack</div><pre class="content">|
-      . $self->get_caller
-      . "</pre>";
-    my $session = "";
+    # Get a sanitised dump of the settings, session and environment
+    $_ = $_ ? dumper($_) : '<i>undefined</i>' for $settings, $session, $env;
 
-    if ( $self->session ) {
-        $session =
-            qq[<div class="title">Session</div><pre class="content">]
-          . dumper( $self->session->data )
-          . "</pre>";
-    }
-    return "$source $settings $session $env";
+    return <<"END_HTML";
+<div class="title">Stack</div><pre class="content">$stack</pre>
+<div class="title">Settings</div><pre class="content">$settings</pre>
+<div class="title">Session</div><pre class="content">$session</pre>
+<div class="title">Environment</div><pre class="content">$env</pre>
+END_HTML
 }
 
 sub get_caller {
@@ -463,6 +455,8 @@ sub _censor {
     my $censored = 0;
     for my $key ( keys %$hash ) {
         if ( ref $hash->{$key} eq 'HASH' ) {
+            # Take a copy of the data, so we can hide sensitive-looking stuff:
+            $hash->{$key} = { %{ $hash->{$key} } };
             $censored += _censor( $hash->{$key} );
         }
         elsif ( $key =~ /(pass|card?num|pan|secret)/i ) {
@@ -503,7 +497,7 @@ Dancer2::Core::Error - Class representing fatal errors
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 SYNOPSIS
 
