@@ -1,6 +1,6 @@
 package Dancer2::Core::Role::Hookable;
 # ABSTRACT: Role for hookable objects
-$Dancer2::Core::Role::Hookable::VERSION = '0.161000';
+$Dancer2::Core::Role::Hookable::VERSION = '0.161000_01';
 use Moo::Role;
 use Dancer2::Core;
 use Dancer2::Core::Types;
@@ -33,7 +33,7 @@ sub _add_postponed_hooks {
 
     # find the internal name of the hooks, from the caller name
     my $caller = ref($self);
-    my ( $dancer, $h_type, $h_name, @rest ) = map {lc} split /::/, $caller;
+    my ( $dancer, $h_type, $h_name, @rest ) = map lc, split /::/, $caller;
     $h_name = $rest[0] if $h_name eq 'role';
     if ( $h_type =~ /(template|logger|serializer|session)/ ) {
         $h_name = $h_type;
@@ -129,7 +129,7 @@ Dancer2::Core::Role::Hookable - Role for hookable objects
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 AUTHOR
 
