@@ -1,6 +1,6 @@
 package Dancer2::Core::Role::Logger;
 # ABSTRACT: Role for logger engines
-$Dancer2::Core::Role::Logger::VERSION = '0.161000';
+$Dancer2::Core::Role::Logger::VERSION = '0.161000_01';
 use Dancer2::Core::Types;
 
 use Moo::Role;
@@ -150,12 +150,12 @@ sub format_message {
 sub _serialize {
     my @vars = @_;
 
-    return join q{}, map {
+    return join q{}, map +(
         ref $_
           ? Data::Dumper->new( [$_] )->Terse(1)->Purity(1)->Indent(0)
           ->Sortkeys(1)->Dump()
           : ( defined($_) ? $_ : 'undef' )
-    } @vars;
+    ), @vars;
 }
 
 sub core {
@@ -197,7 +197,7 @@ Dancer2::Core::Role::Logger - Role for logger engines
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 DESCRIPTION
 
