@@ -1,9 +1,10 @@
 package Dancer2::Core::Role::Serializer;
 # ABSTRACT: Role for Serializer engines
-$Dancer2::Core::Role::Serializer::VERSION = '0.161000';
+$Dancer2::Core::Role::Serializer::VERSION = '0.161000_01';
 use Moo::Role;
 use Try::Tiny;
 use Dancer2::Core::Types;
+use Scalar::Util 'blessed';
 
 with 'Dancer2::Core::Role::Engine';
 
@@ -40,14 +41,14 @@ around serialize => sub {
     $content && length $content > 0
         or return $content;
 
-    $self->execute_hook( 'engine.serializer.before', $content );
+    blessed $self && $self->execute_hook( 'engine.serializer.before', $content );
 
     my $data;
     try {
-        $data = $self->$orig($content, $options);
-        $self->execute_hook( 'engine.serializer.after', $data );
+        $data = $self->$orig( $content, $options );
+        blessed $self && $self->execute_hook( 'engine.serializer.after', $data );
     } catch {
-        $self->log_cb->( core => "Failed to serialize the request: $_" );
+        blessed $self && $self->log_cb->( core => "Failed to serialize the request: $_" );
     };
 
     return $data;
@@ -93,7 +94,7 @@ Dancer2::Core::Role::Serializer - Role for Serializer engines
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 DESCRIPTION
 
