@@ -1,6 +1,6 @@
 package Dancer2::Core::Route;
 # ABSTRACT: Dancer2's route handler
-$Dancer2::Core::Route::VERSION = '0.161000';
+$Dancer2::Core::Route::VERSION = '0.161000_01';
 use Moo;
 use Dancer2::Core::Types;
 use Carp 'croak';
@@ -135,14 +135,15 @@ sub execute {
 
     my $content = $self->code->( $app, @args );
 
-    # a user might have set the content in the response,
-    # so we ignore the return value and use that content instead
-    # but we only override if there was no object returned
+    # users may set content in the response. If the response has
+    # content, and the returned value from the route code is not
+    # an object (well, reference) we ignore the returned value
+    # and use the existing content in the response instead.
     $RESPONSE->has_content && !ref $content
-        and $content = $RESPONSE->content;
+        and return $app->_prep_response( $RESPONSE );
 
     my $type = blessed($content)
-        or return $app->_add_content_to_response( $RESPONSE, $content );
+        or return $app->_prep_response( $RESPONSE, $content );
 
     # Plack::Response: proper ArrayRef-style response
     $type eq 'Plack::Response'
@@ -236,9 +237,12 @@ sub _build_regexp_from_string {
 sub validate_options {
     my ( $self, $request ) = @_;
 
-    while ( my ( $option, $value ) = each %{ $self->options } ) {
+    for my $option ( keys %{ $self->options } ) {
         return 0
-          if ( not $request->$option ) || ( $request->$option !~ $value );
+          if (
+            ( not $request->$option )
+            || ( $request->$option !~ $self->options->{ $option } )
+          )
     }
     return 1;
 }
@@ -257,7 +261,7 @@ Dancer2::Core::Route - Dancer2's route handler
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 ATTRIBUTES
 
