@@ -1,12 +1,11 @@
 package Dancer2::Core::Runner;
 # ABSTRACT: Top-layer class to start a dancer app
-$Dancer2::Core::Runner::VERSION = '0.161000';
+$Dancer2::Core::Runner::VERSION = '0.161000_01';
 use Moo;
 use Carp 'croak';
 use Dancer2::Core::MIME;
 use Dancer2::Core::Types;
 use Dancer2::Core::Dispatcher;
-use HTTP::Server::PSGI;
 use Plack::Builder qw();
 
 # Hashref of configurable items for the runner.
@@ -76,6 +75,7 @@ has timeout => (
 sub _build_server {
     my $self = shift;
 
+    require HTTP::Server::PSGI;
     HTTP::Server::PSGI->new(
         host            => $self->host,
         port            => $self->port,
@@ -259,7 +259,7 @@ Dancer2::Core::Runner - Top-layer class to start a dancer app
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 AUTHOR
 
