@@ -1,6 +1,6 @@
 package Dancer2::Handler::AutoPage;
 # ABSTRACT: Class for handling the AutoPage feature
-$Dancer2::Handler::AutoPage::VERSION = '0.161000';
+$Dancer2::Handler::AutoPage::VERSION = '0.161000_01';
 use Moo;
 use Carp 'croak';
 use Dancer2::Core::Types;
@@ -27,7 +27,7 @@ sub code {
         my $app    = shift;
         my $prefix = shift;
 
-        my $template = $app->engine('template');
+        my $template = $app->template_engine;
         if ( !defined $template ) {
             $app->response->has_passed(1);
             return;
@@ -71,7 +71,7 @@ Dancer2::Handler::AutoPage - Class for handling the AutoPage feature
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 DESCRIPTION
 
