@@ -1,6 +1,6 @@
 package Dancer2::Logger::Capture;
 # ABSTRACT: Capture dancer logs
-$Dancer2::Logger::Capture::VERSION = '0.161000';
+$Dancer2::Logger::Capture::VERSION = '0.161000_01';
 use Moo;
 use Dancer2::Logger::Capture::Trap;
 
@@ -38,7 +38,7 @@ Dancer2::Logger::Capture - Capture dancer logs
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 SYNOPSIS
 
@@ -46,7 +46,7 @@ The basics:
 
     set logger => "capture";
 
-    my $trap = dancer_app->engine('logger')->trapper;
+    my $trap = dancer_app->logger_engine->trapper;
     my $logs = $trap->read;
 
 A worked-out real-world example:
@@ -59,7 +59,7 @@ A worked-out real-world example:
     warning "Danger!  Warning!";
     debug   "I like pie.";
 
-    my $trap = dancer_app->engine('logger')->trapper;
+    my $trap = dancer_app->logger_engine->trapper;
 
     is_deeply $trap->read, [
         { level => "warning", message => "Danger!  Warning!" },
