@@ -1,7 +1,7 @@
 # ABSTRACT: Serializer for handling Dumper data
 
 package Dancer2::Serializer::Dumper;
-$Dancer2::Serializer::Dumper::VERSION = '0.161000';
+$Dancer2::Serializer::Dumper::VERSION = '0.161000_01';
 use Moo;
 use Carp 'croak';
 use Data::Dumper;
@@ -12,15 +12,9 @@ with 'Dancer2::Core::Role::Serializer';
 has '+content_type' => ( default => sub {'text/x-data-dumper'} );
 
 # helpers
-sub from_dumper {
-    my $s = Dancer2::Serializer::Dumper->new;
-    $s->deserialize(@_);
-}
+sub from_dumper { __PACKAGE__->deserialize(@_) }
 
-sub to_dumper {
-    my $s = Dancer2::Serializer::Dumper->new;
-    $s->serialize(@_);
-}
+sub to_dumper { __PACKAGE__->serialize(@_) }
 
 # class definition
 sub serialize {
@@ -56,7 +50,7 @@ Dancer2::Serializer::Dumper - Serializer for handling Dumper data
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 DESCRIPTION
 
