@@ -1,5 +1,5 @@
 package Dancer2::Session::YAML;
-$Dancer2::Session::YAML::VERSION = '0.161000';
+$Dancer2::Session::YAML::VERSION = '0.161000_01';
 # ABSTRACT: YAML-file-based session backend for Dancer2
 
 use Moo;
@@ -39,7 +39,7 @@ Dancer2::Session::YAML - YAML-file-based session backend for Dancer2
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 DESCRIPTION
 
@@ -71,6 +71,7 @@ files in /tmp/dancer-sessions
       session:
         YAML:
           session_dir: "/tmp/dancer-sessions"
+          cookie_duration: 3600    # Default cookie timeout in seconds
 
 =head1 DEPENDENCY
 
