@@ -1,6 +1,6 @@
 package Dancer2::Test;
 # ABSTRACT: Useful routines for testing Dancer2 apps
-$Dancer2::Test::VERSION = '0.161000';
+$Dancer2::Test::VERSION = '0.161000_01';
 use strict;
 use warnings;
 
@@ -74,7 +74,7 @@ sub dancer_response {
         *Dancer2::Core::App::set_request = sub {
             my $self = shift;
             $self->_set_request( $request );
-            $_->set_request( $request ) for $self->defined_engines;
+            $_->set_request( $request ) for @{ $self->defined_engines };
         };
     }
 
@@ -625,7 +625,7 @@ Dancer2::Test - Useful routines for testing Dancer2 apps
 
 =head1 VERSION
 
-version 0.161000
+version 0.161000_01
 
 =head1 SYNOPSIS
 
